require 'TagTreeScanner'

class SimpleMarkup < TagTreeScanner
	@root_factory.allows_text = false
	
	@tag_genres[ :root ] = [ ]

	@tag_genres[ :root ] << TagFactory.new( :paragraph,
		# A line that doesn't have whitespace at the start
		:open_match => /(?=\S)/, :open_requires_bol => true,
		
		# Close when you see a double return
		:close_match => /\n[ \t]*\n/,
		:allows_text => :true,
		:allowed_genre => :inline
	)

	@tag_genres[ :root ] << TagFactory.new( :preformatted,
		# Grab all lines that are indented up until a line that isn't
		:open_match => /((\s+).+?)\n+(?=\S)/m, :open_requires_bol => true,
		:setup => lambda{ |tag, scanner, tagtree|
			# Throw the contents I found into the tag
			# but remove leading whitespace
			tag << scanner[1].gsub( /^#{scanner[2]}/, '' )
		},
		:autoclose => :true
	)

	@tag_genres[ :inline ] = [ ]

	@tag_genres[ :inline ] << TagFactory.new( :bold,
		# An asterisk followed by a letter or number
		:open_match => /\*(?=[a-z0-9])/i,

		# Close when I see an asterisk OR a newline coming up
		:close_match => /\*|(?=\n)/,
		:allows_text => true,
		:allowed_genre => :inline
	)

	@tag_genres[ :inline ] << TagFactory.new( :italic,
		# An underscore followed by a letter or number
		:open_match => /_(?=[a-z0-9])/i,

		# Close when I see an underscore OR a newline coming up
		:close_match => /_|(?=\n)/,
		:allows_text => true,
		:allowed_genre => :inline
	)
end

raw_text = <<ENDINPUT
Hello World! You're _soaking in_ my test.
This is a *subset* of markup that I allow.

Hi paragraph two. Yo! A code sample:

  def foo
    puts "Whee!"
  end

_That, as they say, is that._

ENDINPUT

markup = SimpleMarkup.new( raw_text ).to_xml
puts markup


#=> <paragraph>Hello World! You're <italic>soaking in</italic> my test.
#=> This is a <bold>subset</bold> of markup that I allow.</paragraph>
#=> <paragraph>Hi paragraph two. Yo! A code sample:</paragraph>
#=> <preformatted>def foo
#=>   puts "Whee!"
#=> end</preformatted>
#=> <paragraph><italic>That, as they say, is that.</italic></paragraph>
